function mat=discmat_yee(epsin,width,lambda,symx,symy)
%
%  Sets up discretization matrix for H_x and H_y components with a 
%  diagonal eps-tensor using first-order discretized equations on the
%  Yee mesh.
%
%  Vacuum wave vector
%
k0=2*pi/lambda;
%
%  nx,ny=number of gridpoints along x,y axes
%
ngrid=size(epsin);
nx=ngrid(1);
ny=ngrid(2);
%
%  dx,dy=discretization steps along x,y axes
%
dx=width(1)/nx;
dy=width(2)/ny;
%
%  Calculate a few constants outside the main loop
%
k02=k0^2;
%
%  Total dimension of matrix is 2*ntot by 2*ntot
%
ntot=nx*ny;
%
%  Allocate a sparse matrix of dimension 2*ntot by 2*ntot with
%  at most 14*ntot nonzero elements
%
mat=spalloc(2*ntot,2*ntot,14*ntot);
eps=zeros(nx,ny,3);
%
% Will handle a scalar eps-function (which is translated into a diagonal
% tensor which is epsin times identity matrix), or a eps(:,:,n) function
% with n>2, where the eps-elements will be taken from n=1:3.
%
if (length(ngrid)==2)
 eps(:,:,1)=epsin;
 eps(:,:,2)=epsin;
 eps(:,:,3)=epsin;
elseif (ngrid(3)==3) 
 eps=epsin;
elseif (ngrid(3)>3)
 eps=epsin(:,:,1:3);
end 
for n=1:ny
 for m=1:nx
  if (m<nx)
   epssxm1(m,n)=1/eps(m+1,n,3);
  else
   epssxm1(m,n)=1/eps(m,n,3);
  end
  if (n<ny)
   epssym1(m,n)=1/eps(m,n+1,3);
  else
   epssym1(m,n)=1/eps(m,n,3);
  end
 end
end
f1=ones(nx,ny)/dx^2;
%
%  x-equation
%
dxm2=1/dx^2;
f2=eps(:,:,2)./eps(:,:,3)*dxm2;
f4=(1-eps(:,:,2).*epssym1)*dxm2;
f=k02*eps(:,:,2)-(f2-f4+3*dxm2);
mat=mat+addmat(f,nx,ny,'x','x',0,0,symx,symy);
f=f1-f2;
mat=mat-addmat(f,nx,ny,'x','y',0,0,symx,symy);
mat=mat+addmat(f1,nx,ny,'x','x',-1,0,symx,symy);
mat=mat+addmat(f1,nx,ny,'x','x',1,0,symx,symy);
mat=mat+addmat(f2,nx,ny,'x','x',0,-1,symx,symy);
f=dxm2-f4;
mat=mat+addmat(f,nx,ny,'x','x',0,1,symx,symy);
f=dxm2-f2;
mat=mat+addmat(f,nx,ny,'x','y',-1,0,symx,symy);
mat=mat+addmat(f4,nx,ny,'x','y',0,1,symx,symy);
mat=mat-addmat(f4,nx,ny,'x','y',-1,1,symx,symy);
%
%  y-equation
%
f2=eps(:,:,1)./eps(:,:,3)*dxm2;
f4=(1-eps(:,:,1).*epssxm1)/dx^2;
f=k02*eps(:,:,1)-(3*dxm2+f2-f4);
mat=mat+addmat(f,nx,ny,'y','y',0,0,symx,symy);
f=f1-f2;
mat=mat-addmat(f,nx,ny,'y','x',0,0,symx,symy);
mat=mat+addmat(f1,nx,ny,'y','y',0,-1,symx,symy);
mat=mat+addmat(f2,nx,ny,'y','y',-1,0,symx,symy);
mat=mat+addmat(f1,nx,ny,'y','y',0,1,symx,symy);
f=dxm2-f4;
mat=mat+addmat(f,nx,ny,'y','y',1,0,symx,symy);
f=dxm2-f2;
mat=mat+addmat(f,nx,ny,'y','x',0,-1,symx,symy);
mat=mat+addmat(f4,nx,ny,'y','x',1,0,symx,symy);
mat=mat-addmat(f4,nx,ny,'y','x',1,-1,symx,symy);

function mat=addmat(func,nx,ny,index_left,index_right,shiftx,shifty,sx,sy)
% 
%  Function to add an element to the discretization matrix. The element
% couples H-vector component index_left to component index_right, and the
% shift between xy grid coordinates is (shiftx,shifty). Sx and sy specifies
% boundary conditions at x- and y-axis (left and lower boundaries). For
% sx=0 a zero-boundary condition on the x-axis is assumed, whereas for sx=1,
% H_x is even under reflections in the x-axis (by correspondence, H_y is then
% odd under this reflection). For sx=-1, H_x is odd, and H_y even. For sy=1,
% H_x is even under reflection in the y-axis, whereas it is odd for sy=-1.
% The implementation of the reflection symmetries are only compatible with
% a Yee mesh where (0,0) is an E_z point, H_x points run up along the y-axis,
% and H_y points along the x-axis. 
%
symx=sx;
symy=sy;
if (index_right=='y')
 symx=-symx;
 symy=-symy;
end
ntot=2*nx*ny;
mat=spalloc(ntot,ntot,7*ntot);
if (index_left=='x') 
 coll=1;
else
 coll=2;
end
if (index_right=='x') 
 colr=1;
else
 colr=2;
end
dcol=2*(shiftx+nx*shifty);
dcol_xaxis=0;
dcol_yaxis=0;
if (abs(symx)>0 && shifty<0)
 if (colr==1)
  dcol_xaxis=2*shiftx;
 else
  dcol_xaxis=2*(shiftx-nx*shifty);
 end
end
if (abs(symy)>0 && shiftx<0)
 if (colr==1)
  dcol_yaxis=2*(nx*shifty-shiftx);
 else
  dcol_yaxis=2*nx*shifty;
 end
end
 
if (shiftx<0 && shifty<0 && abs(symx*symy)>0) 
 if (colr==1)
  mat(coll,colr-2*shiftx)=symx*symy*func(1,1);
 else
  mat(coll,colr-2*nx*shifty)=symx*symy*func(1,1);
 end
elseif (shiftx<0 && shifty>=0 && abs(symy)>0)
 if (colr==1)
  mat(coll,colr-2*(shiftx-nx*shifty))=symy*func(1,1);
 else
  mat(coll,colr+2*nx*shifty)=symy*func(1,1);
 end
elseif (shiftx>=0 && shifty<0 && abs(symx)>0)
 if (colr==1)
  mat(coll,colr+2*shiftx)=symx*func(1,1);
 else
  mat(coll,colr+2*(shiftx-nx*shifty))=symx*func(1,1);
 end
end
for n=1:ny
 for m=1:nx
  if (colr+dcol>0 && colr+dcol<=ntot && m+shiftx>0 && m+shiftx<=nx)
   mat(coll,colr+dcol)=func(m,n);
  end
  if (m==1 && shiftx<0 && n>1 && colr+dcol_yaxis<=ntot && abs(symy)>0)
   mat(coll,colr+dcol_yaxis)=symy*func(m,n);   
  end
  if (n==1 && shifty<0 && m>1 && m+shiftx<=ntot && colr+dcol_xaxis<=ntot && abs(symx)>0) 
   mat(coll,colr+dcol_xaxis)=symx*func(m,n);   
  end
  coll=coll+2;
  colr=colr+2;
 end
end
